#include <QApplication>
#include <QWidget>
#include <QCheckBox>
#include <QVBoxLayout>
#include <QResizeEvent>
#include <QStyle>

#include <QWindow>
#include <QScreen>

#include <QDebug>
#include <QSysInfo>

#include <algorithm>
#include <iostream>

static QString title()
{
    return QLatin1String(qVersion()) + QLatin1Char(' ') + QGuiApplication::platformName()
           + QLatin1Char(' ') + qApp->style()->objectName();
}

template <class Str>
static void formatScreen(const QScreen *s, Str & stream)
{
    auto g = s->geometry();
    stream << s->name() << "\", " << g.width() << 'x' << g.height() << Qt::forcesign
           << g.x() << g.y() << Qt::noforcesign << ", dpr=" << s->devicePixelRatio()
           << ", " << s->logicalDotsPerInchX() << "DPI";
}

class Widget : public QWidget
{
public:
    Widget()
    {
        QVBoxLayout *vbox = new QVBoxLayout(this);
        m_cb = new QCheckBox("Test 123", this);
        vbox->addWidget(m_cb);

        m_font.fromString("Noto Sans,-1,18,5,50,0,0,0,0,0,Regular");
        setFont(m_font);

        resize(200, 100);        
    }

    void resizeEvent(QResizeEvent *event) override
    {
        double factor = event->size().width() / 200.0;
        QFont newFont(m_font);
        auto oldPs = m_font.pixelSize();
        auto ps = int(m_font.pixelSize() * factor);
        qDebug() << __FUNCTION__<< event->size() << "factor=" << factor
                 << "pixelSize" << oldPs << "->" << ps << '\n' << newFont;
        newFont.setPixelSize(ps);
        setFont(newFont);
    }

    void screenChange()
    {
        qDebug().noquote().nospace() << '"' << screen()->name() << "\",  DPR=" << devicePixelRatioF() << ' '
           << logicalDpiX() << "DPI";
    }

private:
    QCheckBox *m_cb;
    QFont m_font;
};

int main(int argc, char **argv)
{
    QStringList args;
    std::copy(argv + 1, argv + argc, std::back_inserter(args));

#if QT_VERSION < 0x060000
    if (args.contains("-n")) {
        qDebug() << "AA_DisableHighDpiScaling";
        QCoreApplication::setAttribute(Qt::AA_DisableHighDpiScaling);
    } else {
        qDebug() << "AA_EnableHighDpiScaling";
        QCoreApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
        QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
    }
#endif

#if QT_VERSION >= 0x050E00
    int roundIndex = args.indexOf("-r");
    if (roundIndex >= 0 && roundIndex < args.size() - 1) {
        ++roundIndex;
        bool ok;
        int rpi = args.at(roundIndex).toInt(&ok);
        if (!ok || rpi  < 0 || rpi > int(Qt::HighDpiScaleFactorRoundingPolicy::PassThrough)) {
            qWarning() << "Invalid RoundingPolicy" << args.at(roundIndex);
            return -1;
        }
        auto rp = static_cast<Qt::HighDpiScaleFactorRoundingPolicy>(rpi);
        qDebug() << rp;
        QGuiApplication::setHighDpiScaleFactorRoundingPolicy(rp);
    }
#endif // Qt 5.14


    QApplication app(argc, argv);
    QString t = title() + ' ' + args.join(' ');
    {
        auto dbg = qDebug().noquote().nospace();
        dbg << t << '\n';
#if QT_VERSION >= 0x050E00
        dbg << "highDpiScaleFactorRoundingPolicy=" << QGuiApplication::highDpiScaleFactorRoundingPolicy() << '\n';
#endif // Qt 5.14
        for (auto s : QGuiApplication::screens()) {
            formatScreen(s, dbg);
            dbg << '\n';
        }
    }

    Widget w;
    w.setWindowTitle(t);

    w.show();
    auto wh = w.windowHandle();
    QObject::connect(wh, &QWindow::screenChanged, &w, &Widget::screenChange);
    for (auto s : QGuiApplication::screens())
        QObject::connect(s, &QScreen::logicalDotsPerInchChanged, &w, &Widget::screenChange);

    return app.exec();
}
